/*
 *                   jfjji,.           H A M M E R H E A D  S H A R K W A R E
 *                  ,#######EL:                  http://www.hhsw.de
 *                   .,j######Kj       
 *                    ;W########G:     This  code  is  free software; you can
 *                   j#####G ,D#L:     redistribute it and/or modify it under
 *             .,;;,G#####D.           the  terms  of  the GNU General Public
 *           jE##########G             License   as  published  by  the  Free
 *             .,W######D.             Software  Foundation; either version 2
 *             :L#####K##G             of  the  License,  or (at your option)
 *           .f#####G;.t##:            any later version.
 *         ;L###ELi.    ,D,            
 *       iEEGWj                        This  code  is distributed in the hope
 *      tW;                            that  it  will  be useful, but without
 *     jf                              any warranty; without even the implied
 *    .Kt                              warranty of merchantability or fitness
 *    i:                               for  a particular purpose. See the GNU
 *    ;                                General   Public   License   for  more
 *    .                                details (http://www.gnu.org).
 *
 * This software is based on bbbutton by Konstantinos Mantzaris
 * <GODoal@hotmail.com> with ideas from wmNetscapeKiller by Pierre-Olivier 
 * <pblumo@free.fr>.
 *
 * @author mf@hhsw.de
 * @version 20040712
 */
#include <sys/types.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <iostream.h>
#include <fstream.h>

#include "bbdockrc.hh"

/*
 * Initialize class with application name (argv[0])
 *
 * @param binary - name of the running program
 */
BBDockRC::BBDockRC( char *binary )
{
	int		len;

	if( !(len = strlen( binary )) )
		return;

	for( binary += len; 
		len-- && *(--binary) != '/'; );

	if( *binary == '/' )
		binary++;

	char	*home;

	if( !(home = getenv( "HOME" )) )
		return;

	len = strlen( home )+strlen( binary )+5;

	if( !(rcfilename = new char[len]) )
		return;

	sprintf( rcfilename, "%s/.%src", home, binary );
}

/*
 * Clean up
 */
BBDockRC::~BBDockRC()
{
	if( rcfilename )
		delete rcfilename;
}

/*
 * Load icons from file
 */
BBDock *BBDockRC::load()
{
	ifstream	fin( rcfilename, ios::in );

	if( !fin )
	{	
		cerr << "Unable to open resource configuraton file for reading\n";
		return NULL;
	}

	while( !fin.eof() )
	{
		char	buf[1024];
		BBDock	*dock;

		fin.getline( buf, sizeof( buf ), '\n' );
		
		char	*comment;

		if( (comment = strchr( buf, '#' )) )
			*comment = 0;

		char	*img;
		char	*cmd;

		if( (img = strtok( buf, "\t :" )) &&
			(cmd = strtok( NULL, "\n" )) )
		{
			int	pfd[2];

			if( pipe( pfd ) == -1 )
			{
				cerr << "Pipe failed" << endl;
				return NULL;
			}
		
			switch( fork() )
			{
				case -1:
					cerr << "Could not fork" << endl;;
					return NULL;
				case 0:
					close( pfd[0] );
					fin.close();
					return new BBDock( img, cmd, pfd[1] );
			}

			close( pfd[1] );

			char	signal;

			if( read( pfd[0], &signal, sizeof( char ) ) < 0 )
			{
				cerr << "Pipe broken" << endl;
				return NULL;
			}

			switch( signal )
			{
				default:
					cerr << "Unknown signal caught" << endl;
					return NULL;
				case BBDock::Loaded:
					// proceed loading, this signal transfer
					// guarantees the icons will appear in the
					// right order
					break;
			}
			
			close( pfd[0] );
		}
	}
	
	return NULL;
}
